﻿--[[
Name: LibGuildBankComm-1.0
Revision: $Revision: 1 $
Author(s): Myrroddin (psvander@gmail.com)
Documentation: http://www.wowace.com/index.php/LibResComm-1.0
SVN:  svn://svn.wowace.com/wow/librescomm-1-0/mainline/trunk
Description: Saves guild bank contents and sends/receives updates over the addon channel
Dependencies: LibStub, CallbackHandler-1.0, Ace3
]]

local MAJOR_VERSION = "LibGuildBankComm-1.0"
local MINOR_VERSION = 90000 + tonumber(("$Revision: 1 $"):match("%d+"))

local lib = LibStub:NewLibrary(MAJOR_VERSION, MINOR_VERSION)
if not lib then return end
local lib = LibStub("AceAddon-3.0"):NewAddon("LibGuildBankComm-1.0", "AceComm-3.0", "AceEvent-3.0")

local guilded
local guildName
local gbContents = {}

function lib:OnInitialize()
	if not lib.Callbacks then
		lib.Callbacks = LibStub("CallbackHandler-1.0"):New(lib)
	end
	guilded = IsInGuild("player")
	guildName = GetGuildInfo("player")
	lib:RegisterComm("GuildBankComm")
	-- self:CancelAllTimers()
	self:SendCommMessage("GuildBankComm", "REQUEST", "GUILD")
end

function lib:OnEnable()
	self:RegisterEvent("GUILD_ROSTER_UPDATE")
	self:RegisterEvent("PLAYER_GUILD_UPDATE")
	self:RegisterEvent("GUILDBANKBAGSLOTS_CHANGED")
	self:RegisterEvent("GUILDBANKFRAME_OPENED")
	self:RegisterEvent("GUILDBANKFRAME_CLOSED")
	self:RegisterEvent("GUILDBANK_UPDATE_MONEY")
	self:RegisterEvent("GUILDBANK_UPDATE_TABS")
end

function lib:OnDisable()
	self:UnRegisterAllEvents()
end

function lib:OnCommReceived(prefix, msg, distribution, sender)
	if prefix ~= "GuildBankComm" then return end
	if sender == "player" then return end
	
	for cmd in msg:gmatch("(%a+)%s?([^#]*)") do
		if cmd == "REQEST" then
			self:SendCommMessage("GuildBankComm", localTimeStamp, sender)
		elseif cmd == "SYNC" then
			self:SendCommMessage("GuildBankCOmm", gbContents, sender)
		elseif cmd == "UPDATE" then
			gbContents = msg
			-- lib.CallBacks:Fire("GuildBankComm_ContentsChanged", unit, gbContents, self:TimeStamp)
		end
	end
end

function lib:PLAYER_GUILD_UPDATE(unitID)
	if unitID ~= "player" then return end
	if guilded ~= 1 then
		wipe(gbContents)
	end
	guildName = GetGuildInfo("player")
	guilded = IsInGuild("player")
end

--[[
function lib:GUILD_ROSTER_UPDATE(unit)
	local memberCount = GetNumGuldMembers()
	if unit == "player" and guilded ~= 1 then
		wipe(gbContents)
		return
	end
	if (memberCount > numGuildMembers) then
		-- people joined guild
		local contents = tostring(gbContents)
		lib:SendCommMessage("GuildBankComm", contents, "GUILD")
		lib.CallBacks:Fire("GuildBankComm_RosterUpdate", unit, contents)
		numGuildMembers = memberCount
	end
	memberCount = nil
end
]]--

function lib:GUILDBANK_UPDATE_MONEY(unit)
	if unit ~= "player" then return end
	gbContents[guildName].Money = self:GetGuildBankMoneyString()
	--[[
	local contents = tostring(gbContents)
	lib:SendCommMessage("GuildBankComm", contents, "GUILD")
	lib.CallBacks:Fire("GuildBankComm_ContentsChanged", unit, contents)
	]]--
end

function lib:GUILDBANKFRAME_OPENED()
	gbContents[guildName].numTabs = GetNumGuildBankTabs()
	gbContents[guildName].Money = self:GetGuildBankMoneyString()
	
	local name, icon, view	
	for page = 1, gbContents[guildName].numTabs do
		name, icon, view = GetGuildBankTabInfo(page)
		gbContents[guildName].tab.texture[page] = icon
		gbContents[guildName].tab.name[page] = name
		gbContents[guildName].tab.view[page] = view
	end
end

function lib:GUILDBANKFRAME_CLOSED()
	self:SendCommMessage("GuildBankComm", "UPDATE", "GUILD")
	-- lib.CallBacks:Fire("GuildBankComm_ContentsChanged", unit, gbContents, self:TimeStamp)
end

function lib:GUILDBANK_UPDATE_TABS()
	gbContents[guildName].numTabs = GetNumGuildBankTabs()
end

function lib:GUILDBANKBAGSLOTS_CHANGED()
	if (GuildBankFrame and GuildBankFrame:IsVisible()) or (BagnonFrameguildbank and BagnonFrameguildbank:IsVisible()) then
		local page = GetCurrentGuildBankTab()
		--98 slots on a page
		for slot = 1, 98 do
			--see if the slot has an item in it
			if GetGuildBankItemLink(page, slot) then
				--if there is link, parse the ID and place that in table w/ key = slot
				_, _, itemString = string.find(GetGuildBankItemLink(page, slot), "^|c%x+|H(.+)|h%[.+%]")
				gbContents[guildName].links[page][slot] = itemString
				
				--get item info too, need for stack sizes in image
				_, num = GetGuildBankItemInfo(page, slot)
				gbContents[guildName].info[page][slot] = num
			else
				--if there is no link, slot is empty... fill slot w/ empty string
				--needed to keep consistent tables with keys from 1 to 98
				gbContents[guildName].links[page][slot] = "empty"
				gbContents[guildName].info[page][slot] = "empty"
			end
			num, itemString = nil, nil
		end
		
		--save a timestamp
		local date = GetDate()
		gbContents[guildName].lastScan = string.format("%s %s %s", date, L["at"], self:TimeStamp())
	end
end

function lib:TimeStamp()
	local hour, minute = GetGameTime()
	if minute < 10 then
		minute = "0"..minute
	end
	
	if hour == 0 then
		return string.format(" %s:%s %s", "12", minute, L["AM"])
	elseif hour == 12 then
		return string.format(" %s:%s %s", hour, minute, L["PM"])
	elseif hour > 12 then
		return string.format(" %s:%s %s", (hour-12), minute, L["PM"])
	else
		return string.format(" %s:%s %s", hour, minute, L["AM"])
	end
end

local goldAbrv = L["g"]
local silverAbrv = L["s"]
local copperAbrv = L["c"]
function lib:GetMoneyString()
	local copper = GetGuildBankMoney()
	local gold = (copper - (copper % 10000)) / 10000
	copper = copper % 10000
	local silver = (copper - (copper % 100)) / 100
	copper = copper % 100
	
	return string.format("%s|cff%s%s|r %s|cff%s%s|r %s|cff%s%s|r", gold, "ffd700", goldAbrv, silver, "c7c7cf", silverAbrv, copper, "eda55f", copperAbrv)
end